function [ Appr_Error_Mean,Appr_Error,Alph ,Norm1Alph ,SparsityAlph] = ...
         MAE_M( TrImage, Dic,Cof_Lambda,patch_size,Outdisplay)
     
% MAE_M computes the approximation error for the matrix formulation
% i.e., how well we can represent
% the exact image in the cone defined by the dictionary
%  the distance of the exact image x^{exact} to its projection on the cone
% C , we compute the solutions to the q approximation problems for all
% blocks j = 1, 2,..,q in x^{exact},
% This code handle various dictionaries for a range of values for lambda.
% This codes uses the TFOCS optimization solver (http://cvxr.com/tfocs/)
% make sure TFOCS is properly installed on your machine

% Usage:
%        [ Appr_Error_Mean,Appr_Error,Alph ,Norm1Alph ,SparsityAlph] = ...
%         MAE_M( TrImage, Dic,Cof_Lambda,patch_size);

% Inputs:
% TrImage: The test image, options are: 'Peppers', 'Matches', 
% 'Binary', 'Zirconium', 'Steel', 'D53'
% Dic: The given(learned) dictionary, note that the dictionary should be
% obtained from a training image similar to the test image
% Cof_Lambda: The regularization parameter in the dictionary learning
% problem formulation, note that it should be consistence with the 
% dictionary Dic
% patch_size: The patch sizes of the dictionary elements, should be
% provided if the dictionaries patch sizes are rectangular
% default = [ceil(sqrt(size(D,1))),ceil(sqrt(size(D,1)))]
% Outdisplay: The display option on the screen, when set to 0 no display 
% on the screen, when set to 1 printes progress of the algorithm
% default = 0,

% Outputs:
% Appr_Error_Mean: Mean approximation error
% Appr_Error: The vector of approximation error for each block in the image
% Alph: The best representation/approximation of the jth block in the cone.
% Norm1Alph: ||Alph(j)||_1 for each block j
% SparsityAlph: The sparsity percentage of the representation vector
% (solution) Alph for each block

% Copyright (C) Sara Soltani and Technical University of Denmark, 
% August 2015, ssol@dtu.dk, sarahsoltani@gmail.com

%% The test images

switch(TrImage)
   case 'Peppers'
        IMAGE = imread('Peppers.tif');
        I = double(IMAGE);
        I = (I - min(I(:)))./(max(I(:)) - min(I(:))); 
        TrI = I(601:800,601:800);
        TrI = flipud(TrI);
   case 'Matches'
        IMAGE = imread('Matches.tif');
        I= double(IMAGE);
        I = (I - min(I(:)))./(max(I(:)) - min(I(:))); 
        TrI = I(601:800,401:600);
        TrI = flipud(TrI);
   case 'Binary'
         IMAGE = imread('BinaryTrI.mat');
         TrI= double(IMAGE);
   case 'Steel'
         IMAGE = imread('Steel.tif');
         I= double(IMAGE);
         I=(I-min(I(:)))./(max(I(:))-min(I(:)));
         TrI = I(201:720,201:720);
         TrI = flipud(TrI);
    case 'Zirconium'
          IMAGE = imread('Zirconium.tif');
          I= double(IMAGE);
          I = filter2(fspecial('average',3),I);
          I=(I-min(I(:)))./(max(I(:))-min(I(:)));
          TrI = imrotate(I,5);
          TrI = TrI(201:720,201:720);
    case 'D53'
          IMAGE = imread('D53.tif');
          I= double(IMAGE);
          I=(I-min(I(:)))./(max(I(:))-min(I(:)));
          TrI=I(201:400,201:400);
    otherwise
    warning('Please enter the correct name of the test image.')   
         
end

if nargin < 5 || isempty(Outdisplay)
    Outdisplay = 0;
end

if nargin < 4 || isempty( patch_size )
    patch_size = [ceil(sqrt(size(Dic,1))),ceil(sqrt(size(Dic,1)))];
end 

noc = size(Dic,2);  

Nr_Lambda = length(Cof_Lambda);

%% Extracting all the possible non-overlapping patched in the exact image

[M,N]=size(TrI);

Ncol=N/patch_size(2);
Nrow=M/patch_size(1);

X=zeros(prod(patch_size),Nrow*Ncol);


for i=1:Nrow
    for j=1:Ncol
        patch=TrI((i-1)*patch_size(1)+1:i*patch_size(1),...
            (j-1)*patch_size(2)+1:j*patch_size(2));
        X(:,(i-1)*Ncol+j)=patch(:)';
    end
end
Nr_patches=size(X,2);


Alph = zeros(noc,Nr_patches,Nr_Lambda);
Norm1Alph = zeros(Nr_patches,Nr_Lambda);
SparsityAlph =  zeros(Nr_patches,Nr_Lambda);
Appr_Error = zeros(Nr_patches,Nr_Lambda);
Appr_Error_Mean = zeros(Nr_Lambda,1);

opts.maxIts=100000;
opts.tol=1e-7;
x_0=zeros(noc,1);
if Outdisplay==0
    opts.printEvery = 0;
end

for k=1:Nr_Lambda
    D = Dic(:,:,k);
    for j=1:Nr_patches
    
        alph = tfocs(smooth_quad, { D, -X(:,j) },proj_Rplus,x_0,opts);
         
        Alph(:,j,k)=alph;
        Appr_Error(j,k)=norm(D*alph-X(:,j),2); % Error norm
        Norm1Alph(j,k) = norm(alph,1);
        SparsityAlph(j,k) =100-((length(find(alph)))/length(alph))*100;
        
    end
  
    Appr_Error_Mean(k,1) = mean(Appr_Error(:,k));


end

end

