function [TrI,X_sol,Alph,Error_Sol,tau,delta,Sparsity,Resudial_Fit,...
    AlphaNorm1,AlphaNorm1Avr,t_end] = ...
RecT_Algorithm(TrImage,Dic,N_p,Ra,rnl,geo,patch_size,Reg,Outdisplay)

% Function RecT_Algorithm solves the tomographic reconstruction problem
% in the tensor form for the asked test image by TFOCS solver.
% The tomographic scenario and test images are set up by this function.
% The user can easily modify the function for
% other regularization parameters tau and delta and test problems.
% This code handles one dictionary for a specific lambda.
% The code needs to be modified if one wants to solve the reconstruction
% problem for various values of lambda at the same time

% Usage:
% [TrI,X_sol,Alph,Error_Sol,Sparsity,Resudial_Fit,AlphaNorm1,...
%    AlphaNorm1Avr,t_end] =
%    RecT_Algorithm(TrImage,Dic,patch_size,N_p,Ra,rnl,geo,Outdisplay)

% Inputs:
% TrImage: The test image, options are: 'Peppers', 'Matches', 'Binary',
% 'Zirconium', 'Steel', 'D53'
% Dic: The given(learned) dictionary, note that the dictionary should be
% obtained from a training image similar to the test image
% N_p: Number of tomographic projections 
% Ra: Range of the angles for the tomographic projections, e.g., full-range
% [0, 180], limited-angle [0 180]
% rnl: Gaussian additive noise level in the tomographic data
% geo: Choose a parallel-beam CT geometry or a fan-beam CT geometry,
% options= 'fanbeam' and 'parallelbeam'. default value = 'parallelbeam'
% patch_size: The patch sizes of the blocks in the image, note that 
% the image size is a multiple of the patch size. 
% Default: [size(D,1),size(D,3)]
% Reg: Regularization on the reprsentation in the dictionary. Options are
% sparsity prior (l1 norm regularization) and sparsity+low-rank   
% (l1+nuclear norm regularization), options are 'l1pos' and 'nuclearl1pos'
% Default: 'nuclearl1pos'
% Outdisplay: The display option on the screen, when set to 0 no display 
% on the screen, when set to 1 printes progress of the algorithm
% default = 0,

% Outputs:
% TrI: The exact image considered in the tomographic problem. The size of
% the TrI is predefined.
% X_sol: The tomographic reconstruction solution recovered from Alpha
% Alpha: The sparse representation/solution in the given dictionary
% obtained by solving our tomographic reconstruction problem
% Error_Sol: Relative reconstruction error
% tau: The sparsity regularization parameter
% delta: The block artifacts regularization parameter
% Sparsity: The sparsity percentage of the representation vector Alpha
% Resudial_Fit: The residual of the tomographic data fitting term
% AlphaNorm1: ||Alpha||_1
% AlphaNorm1Avr: ||Alpha||_1/q
% t_end: total time used by the TFOCS to find the representation vector 
% (Alpha) for a specific regularization parameter (tau and delta)

% Copyright (C) Sara Soltani and Technical University of Denmark, 
% August 2015, ssol@dtu.dk, sarahsoltani@gmail.com

global C P D Nr_Blocks

%% The test images

switch(TrImage)
   case 'Peppers'
        IMAGE = imread('Peppers.tif');
        I = double(IMAGE);
        I = (I - min(I(:)))./(max(I(:)) - min(I(:))); 
        TrI = I(601:800,601:800);
        TrI = flipud(TrI);
   case 'Matches'
        IMAGE = imread('Matches.tif');
        I= double(IMAGE);
        I = (I - min(I(:)))./(max(I(:)) - min(I(:))); 
        TrI = I(601:800,401:600);
        TrI = flipud(TrI);
   case 'Binary'
         IMAGE = imread('BinaryTrI.tif');
         TrI= double(IMAGE);
   case 'Steel'
         IMAGE = imread('Steel.tif');
         I= double(IMAGE);
         I=(I-min(I(:)))./(max(I(:))-min(I(:)));
         TrI = I(201:720,201:720);
         TrI = flipud(TrI);
    case 'Zirconium'
          IMAGE = imread('Zirconium.tif');
          I= double(IMAGE);
          I=(I-min(I(:)))./(max(I(:))-min(I(:)));
          I = filter2(fspecial('average',3),I);
          TrI = imrotate(I,5);
          TrI = TrI(201:720,201:720);
    case 'D53'
          IMAGE = imread('D53.tif');
          I= double(IMAGE);
          I=(I-min(I(:)))./(max(I(:))-min(I(:)));
          TrI=I(201:400,201:400);
    otherwise
    warning('Please enter the correct name of the test image.')   
         
end
%% 
rng('default')
rng(500);      % Random seed

if nargin < 9 || isempty(Outdisplay)
    Outdisplay = 0;
end

if nargin < 8 || isempty(Reg)
    Reg = 'nuclearl1pos';
end

if nargin < 7 || isempty(patch_size)
    patch_size = [size(Dic,1),size(Dic,3)];
end 
if nargin < 6 || isempty(geo)
    geo = 'parallelbeam';
end 
%%
[M,N]=size(TrI);
X = TrI(:);

Nr_Blocks = (M/patch_size(1))*(N/patch_size(2));  
[ P ] = Perm_Vec_Tensor( M,N,patch_size );        
IP=zeros(length(P),1);
IP(P)=1:length(P);
[ L ] = L_Matrix( M,N,patch_size);

% The dictionary
D = Dic;
[~,n,p] = size(D);     


%% generating tomographic data


theta = linspace(0,Ra,N_p); theta = theta(2:end); 

switch(geo)
    case 'fanbeam'
        [A] = fanbeamtomo(N,theta);
    case  'parallelbeam'
        [A] = paralleltomo(N,theta);
end

b_exact=A*X;
e = randn(size(b_exact));
e = e/norm(e);
e = rnl*norm(b_exact)*e;
b1 = b_exact+e; % perturbing the data

b2=zeros(size(L,1),1);
b=[b1;b2];

%% Reconstruction with different values of Tau and delta

% Set up regularization parameter for tau and delta
Nr_Tau = 1;
tau = 0.0215;

Nr_Delta = 1;
delta = 10;

%
t_end = zeros(Nr_Delta,Nr_Tau);
Resudial_Fit = zeros(Nr_Delta,Nr_Tau);
Error_Sol = zeros(Nr_Delta,Nr_Tau);
Sparsity = zeros(Nr_Delta,Nr_Tau);
X_sol = zeros (M*N,Nr_Delta,Nr_Tau);
Alpha = zeros(size(D,2)*Nr_Blocks,p,Nr_Delta,Nr_Tau);
AlphaNorm1 = zeros(Nr_Delta,Nr_Tau);
AlphaNorm1Avr = zeros(Nr_Delta,Nr_Tau);

%%  Setting up the problem formulation
gamm=sqrt(size(A,1));
Lss=sqrt(size(L,1));
b=1/gamm*b;

opts.maxIts=100000;
opts.tol=1e-7;
if Outdisplay==0
    opts.printEvery = 0;
end

for k = 1:Nr_Tau
    for i = 1:Nr_Delta
        C=[1/gamm*A;(delta(i)/Lss).*L];
        
        % OPTIMIZATION with TFOCS
        % TFOCS Structure
        % [ x, out, opts ] = tfocs(smoothF, affineF, projectorF, x0, opts)
        x_0 = zeros(n*Nr_Blocks,p); 

        t0=cputime;
        switch(Reg) 
            case 'nuclearl1pos'
                tic
                Alph = tfocs( smooth_quad, { @Linear_Opr_Tensor, -b }, ...
                prox_nuclearposl1( tau(k) ), x_0, opts);
                if Outdisplay==1
                   toc
                end
            case 'l1pos'
                tic
                Alph = tfocs( smooth_quad, { @Linear_Opr_Tensor, -b }, ...
                prox_l1pos( tau(k) ), x_0, opts);
                if Outdisplay==1
                   toc
                end
        end  
        t_end(i,k)=cputime-t0;
        
        AL = reshape(Alph, n, Nr_Blocks , p);
        Z = tprod(D,AL);
        z = Z(:);
        
        X_sol(:,i,k) = z(IP);

        Alpha(:,:,i,k) = Alph; 
        Resudial_Fit(i,k) = norm(A*X_sol(:,i,k)-b1)/norm(b1);
        Error_Sol(i,k) = norm(X_sol(:,i,k)-X)/norm(X);
        Sparsity(i,k) = 100-(length(find(Alph(:)))/length(Alph(:))*100);
        AlphaNorm1(i,k) = norm(Alph(:),1);
        AlphaNorm1Avr(i,k)=1/Nr_Blocks*AlphaNorm1(i,k);
    end
end


end